package projects.dream5;

import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Set;

import javax.naming.OperationNotSupportedException;

import de.jstacs.classifier.ScoreBasedPerformanceMeasureDefinitions;
import de.jstacs.data.Sample;
import de.jstacs.data.Sequence;
import de.jstacs.utils.DoubleList;
import de.jstacs.utils.REnvironment;
import de.jstacs.utils.ToolBox;


/**
 * 
 * @author Jan Grau
 */
public class KMerStatistics {

	private Sample data;
	public WeightedKMers kmers;
	private boolean rc;
	private int off;
	private int k;
	private boolean useMeanForCC;
	
	/**
	 * 
	 * @param data the Sample containing the reference data
	 * @param truth the reference weights (intensities) for data
	 * @param rc if sequence and reverse complement should be identified
	 * @param off the offset from the right for extracting k-mers
	 * @param k the length of the k-mers
	 * @param useMeanForCC if the mean instead of median is used for the correlation coefficient
	 * @throws OperationNotSupportedException 
	 */
	public KMerStatistics(Sample data, double[] truth, boolean rc, int off, int k, boolean useMeanForCC) throws OperationNotSupportedException{
		this.data = data;
		this.k = k;
		this.useMeanForCC = useMeanForCC;
		this.rc = rc;
		this.off = off;
		this.kmers = new WeightedKMers( data, truth );
	}
	
	
	public void fillStatistics( double[] signal, LinkedList<Sequence> positives, DoubleList results ) throws Exception{
		
		WeightedKMers test = new WeightedKMers( data, signal );
		
		Set<Sequence> keys = kmers.list.keySet();
		
		HashMap<Sequence, Boolean> flags = new HashMap<Sequence, Boolean>();
		if( positives != null ) {

			for(Sequence seq : keys){
				flags.put( seq, false );
			}
			
			for(Sequence seq : positives){
				if(rc){
					Sequence rc = //seq.complement();//
						//seq.reverse();
						seq.reverseComplement();
					if(rc.compareTo( seq ) < 0){
						seq = rc;
					}
				}
				flags.put( seq, true );
			}
		}
		
		double[] truth = new double[keys.size()];
		double[] pred = new double[truth.length];
		
		DoubleList posScores = new DoubleList();
		DoubleList negScores = new DoubleList();
		
		double[] testVal;
		int i=0;
		for(Sequence seq : keys){
			
			testVal = test.list.get( seq );
			
			if( positives != null ) {
				if(flags.get( seq )){
					posScores.add( testVal[0] );
				}else{
					negScores.add( testVal[0] );
				}
			}
			
			truth[i] = kmers.list.get( seq )[useMeanForCC ? 1 : 0];
			pred[i] = testVal[useMeanForCC ? 1 : 0];
			
			i++;
		}
		
		if( positives != null ) {
			double[] posScoresAr = posScores.toArray();
			Arrays.sort( posScoresAr );
			double[] negScoresAr = negScores.toArray();
			Arrays.sort( negScoresAr );
			
			results.add( ScoreBasedPerformanceMeasureDefinitions.getAUC_PR( posScoresAr, negScoresAr, null ) );
			results.add( ScoreBasedPerformanceMeasureDefinitions.getAUC_ROC( posScoresAr, negScoresAr, null ) );
		} else {
			results.add( Double.NaN );
			results.add( Double.NaN );
		}
		results.add( ToolBox.pearsonCorrelation( truth, pred ) );
		
	}
	
	
	public class WeightedKMers{
		
		public HashMap<Sequence, double[]> list;
		
		private WeightedKMers(Sample data, double[] truth) throws OperationNotSupportedException{
			
			HashMap<Sequence, DoubleList> map = new HashMap<Sequence, DoubleList>();
			
			Sequence sub;
			DoubleList temp;
			for(int i=0;i<data.getNumberOfElements();i++){
				Sequence seq = data.getElementAt( i );
				for(int j=0;j<seq.getLength()-off-k+1;j++){
					sub = seq.getSubSequence( j, k );
					if(rc){
						Sequence rc = sub.reverseComplement();
							//sub.reverse();
						//System.out.println(rc.getClass());
							//sub.complement();
						if(rc.compareTo( sub ) < 0){
							sub = rc;
						}
					}
					if(map.get( sub ) == null){
						map.put( sub, new DoubleList() );
					}
					temp = map.get( sub );
					temp.add( truth[i] );
				}
			}
			
			Set<Sequence> keys = map.keySet();
			//System.out.println(keys.size());
			
			list = new HashMap<Sequence, double[]>();

			for(Sequence seq : keys){
				temp = map.get( seq );

				list.put( seq, new double[]{temp.median( 0, temp.length() ), temp.mean( 0, temp.length() )} );
			}
		}
		
	}
	
}
